;;;
;;;	The Balsa Asynchronous Hardware Synthesis System
;;;	Copyright (C) 1995-2003 Department of Computer Science
;;;	The University of Manchester, Oxford Road, Manchester, UK, M13 9PL
;;;	
;;;	This program is free software; you can redistribute it and/or modify
;;;	it under the terms of the GNU General Public License as published by
;;;	the Free Software Foundation; either version 2 of the License, or
;;;	(at your option) any later version.
;;;	
;;;	This program is distributed in the hope that it will be useful,
;;;	but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;	GNU General Public License for more details.
;;;	
;;;	You should have received a copy of the GNU General Public License
;;;	along with this program; if not, write to the Free Software
;;;	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;;;
;;;	`net-modify.scm'
;;;	Netlist in-place modification procedures
;;;
;;;	$Id: net-modify.scm,v 1.4 2003/02/08 19:39:44 bardslea Exp $
;;;

;;; net-circuit-decl-remove-instances!: remove the instance list from a cell (in place)
;;;		and replace it with (instances)
(define net-circuit-decl-remove-instances! (lambda (cell)
	(let* ((instances (net-circuit-decl:instances cell)))
		(set-cdr! instances '())
	)
))

;;; net-circuit-decl-add-global-ports!: add the port list `ports-to-add' to the port
;;;		list of the given cell (in place) and adjust the attributes of that cell.
;;;		FIXME, doesn't yet check for pre-existing 'global-ports attribute and merge
;;;		names.
(define net-circuit-decl-add-global-ports! (lambda (cell ports-to-add)
	(let*
		((ports (net-circuit-decl:ports cell))
		 (attributes (net-circuit-decl:attributes cell))
		 (global-attributes
			(list (cons 'global-ports (map car ports-to-add)))
		 )
		)
		(set-cdr! (end-of-list ports) ports-to-add)
		(if attributes
			(set-cdr! (end-of-list attributes) global-attributes)
			(set-cdr! (end-of-list cell) (list (cons 'attributes global-attributes)))
		)
	)
))

;;; balsa-netlist-add-globals!: add global ports to netlist cells
(define balsa-netlist-add-globals! (lambda (netlist)
	(let*
		((globals (make-hash-table 57)))
		(for-each (lambda (cell)
			(let ((global-ports (net-circuit-decl-find-attribute cell 'global-ports)))
				(if global-ports ; this is the start of a global signal
					(begin ; make a note
						(hash-set! globals (net-circuit-decl:name cell)
							(map (lambda (port-name) (net-find-net (net-circuit-decl:ports cell) port-name))
								(cdr global-ports))
						)
					)
					(let ; instances have any globals?
						((all-instance-globals (foldl (lambda (new-globals instance)
							(let ((instance-globals (hash-ref globals (net-instance:name instance))))
								(if instance-globals
									(begin ; add connections
										(set-cdr! (end-of-list (net-instance:instance-connections instance))
											(map car instance-globals))
										(unsorted-uniq (lambda (p1 p2)
											(string=? (net-port:name p1) (net-port:name p2)))
											(append instance-globals new-globals))
									)
									new-globals
								)
							)
						) '() (cdr (net-circuit-decl:instances cell)))))
						(if (not (null? all-instance-globals))
							(begin ; munge the ports of this cell and make a note
								(hash-set! globals (net-circuit-decl:name cell) all-instance-globals)
								(set-cdr! (end-of-list (net-circuit-decl:ports cell)) all-instance-globals)
							)
						)
					)
				)
			)
		) netlist)
	)
))
